<?php
require_once(Mage::getModuleDir('',"Monext_Payline") . '/lib/vendor/autoload.php');
class Monext_Payline_Helper_Data extends Mage_Core_Helper_Data
{

    const SECURITY_MODE                      = 'SSL';
    const CREATE_INVOICE_SHOP_RETURN         = 'return';
    const WALLET_NONE                        = 'NONE';
    const WALLET_3DS                         = '3DS';
    const WALLET_CVV                         = 'CVV';
    const WALLET_BOTH                        = 'BOTH';

    // https://docs.payline.com/display/DT/Historique+de+version+de+l%27API
    const VERSION                            = 22;

    /**
     * Currency codes (ISO 4217) supported by Payline
     * @var array
     */
    protected $_supportedCurrencyCodes       = array(
        'ALL' => '8', // Lek
        'DZD' => '12', // Algerian Dinar
        'ARS' => '32', // Argentine Peso
        'AUD' => '36', // Australian Dollar
        'BSD' => '44', // Bahamian Dollar
        'BHD' => '48', // Bahraini Dinar
        'BDT' => '50', // Taka
        'AMD' => '51', // Armenian Dram
        'BBD' => '52', // Barbados Dollar
        'BMD' => '60', // Bermudian Dollar (customarily known as Bermuda Dollar)
        'BTN' => '64', // Ngultrum
        'BOB' => '68', // Boliviano
        'BWP' => '72', // Pula
        'BZD' => '84', // Belize Dollar
        'SBD' => '90', // Solomon Islands Dollar
        'BND' => '96', // Brunei Dollar
        'MMK' => '104', // Kyat
        'BIF' => '108', // Burundi Franc
        'KHR' => '116', // Riel
        'CAD' => '124', // Canadian Dollar
        'CVE' => '132', // Cape Verde Escudo
        'KYD' => '136', // Cayman Islands Dollar
        'LKR' => '144', // Sri Lanka Rupee
        'CLP' => '152', // Chilean Peso
        'CNY' => '156', // Yuan Renminbi
        'COP' => '170', // Colombian Peso
        'KMF' => '174', // Comoro Franc
        'CRC' => '188', // Costa Rican Colon
        'HRK' => '191', // Croatian Kuna
        'CUP' => '192', // Cuban Peso
        'CYP' => '196', // Cyprus Pound
        'CZK' => '203', // Czech Koruna
        'DKK' => '208', // Danish Krone
        'DOP' => '214', // Dominican Peso
        'SVC' => '222', // El Salvador Colon
        'ETB' => '230', // Ethiopian Birr
        'ERN' => '232', // Nakfa
        'EEK' => '233', // Kroon
        'FKP' => '238', // Falkland Islands Pound
        'FJD' => '242', // Fiji Dollar
        'DJF' => '262', // Djibouti Franc
        'GMD' => '270', // Dalasi
        'GHC' => '288', // Cedi
        'GIP' => '292', // Gibraltar Pound
        'GTQ' => '320', // Quetzal
        'GNF' => '324', // Guinea Franc
        'GYD' => '328', // Guyana Dollar
        'HTG' => '332', // Gourde
        'HNL' => '340', // Lempira
        'HKD' => '344', // Hong Kong Dollar
        'HUF' => '348', // Forint
        'ISK' => '352', // Iceland Krona
        'INR' => '356', // Indian Rupee
        'IDR' => '360', // Rupiah
        'IRR' => '364', // Iranian Rial
        'IQD' => '368', // Iraqi Dinar
        'ILS' => '376', // New Israeli Sheqel
        'JMD' => '388', // Jamaican Dollar
        'JPY' => '392', // Yen
        'KZT' => '398', // Tenge
        'JOD' => '400', // Jordanian Dinar
        'KES' => '404', // Kenyan Shilling
        'KPW' => '408', // North Korean Won
        'KRW' => '410', // Won
        'KWD' => '414', // Kuwaiti Dinar
        'KGS' => '417', // Som
        'LAK' => '418', // Kip
        'LBP' => '422', // Lebanese Pound
        'LSL' => '426', // Loti
        'LVL' => '428', // Latvian Lats
        'LRD' => '430', // Liberian Dollar
        'LYD' => '434', // Libyan Dinar
        'LTL' => '440', // Lithuanian Litas
        'MOP' => '446', // Pataca
        'MWK' => '454', // Kwacha
        'MYR' => '458', // Malaysian Ringgit
        'MVR' => '462', // Rufiyaa
        'MTL' => '470', // Maltese Lira
        'MRO' => '478', // Ouguiya
        'MUR' => '480', // Mauritius Rupee
        'MXN' => '484', // Mexican Peso
        'MNT' => '496', // Tugrik
        'MDL' => '498', // Moldovan Leu
        'MAD' => '504', // Moroccan Dirham
        'OMR' => '512', // Rial Omani
        'NAD' => '516', // Namibian Dollar
        'NPR' => '524', // Nepalese Rupee
        'ANG' => '532', // Netherlands Antillian Guilder
        'AWG' => '533', // Aruban Guilder
        'VUV' => '548', // Vatu
        'NZD' => '554', // New Zealand Dollar
        'NIO' => '558', // Cordoba Oro
        'NGN' => '566', // Naira
        'NOK' => '578', // Norwegian Krone
        'PKR' => '586', // Pakistan Rupee
        'PAB' => '590', // Balboa
        'PGK' => '598', // Kina
        'PYG' => '600', // Guarani
        'PEN' => '604', // Nuevo Sol
        'PHP' => '608', // Philippine Peso
        'GWP' => '624', // Guinea-Bissau Peso
        'QAR' => '634', // Qatari Rial
        'ROL' => '642', // Old Leu
        'RUB' => '643', // Russian Ruble
        'RWF' => '646', // Rwanda Franc
        'SHP' => '654', // Saint Helena Pound
        'STD' => '678', // Dobra
        'SAR' => '682', // Saudi Riyal
        'SCR' => '690', // Seychelles Rupee
        'SLL' => '694', // Leone
        'SGD' => '702', // Singapore Dollar
        'SKK' => '703', // Slovak Koruna
        'VND' => '704', // Dong
        'SIT' => '705', // Tolar
        'SOS' => '706', // Somali Shilling
        'ZAR' => '710', // Rand
        'ZWD' => '716', // Zimbabwe Dollar
        'SZL' => '748', // Lilangeni
        'SEK' => '752', // Swedish Krona
        'CHF' => '756', // Swiss Franc
        'SYP' => '760', // Syrian Pound
        'THB' => '764', // Baht
        'TOP' => '776', // Pa'anga
        'TTD' => '780', // Trinidad and Tobago Dollar
        'AED' => '784', // UAE Dirham
        'TND' => '788', // Tunisian Dinar
        'TMM' => '795', // Manat
        'UGX' => '800', // Uganda Shilling
        'MKD' => '807', // Denar
        'EGP' => '818', // Egyptian Pound
        'GBP' => '826', // Pound Sterling
        'TZS' => '834', // Tanzanian Shilling
        'USD' => '840', // US Dollar
        'UYU' => '858', // Peso Uruguayo
        'UZS' => '860', // Uzbekistan Sum
        'VEB' => '862', // Bolivar
        'WST' => '882', // Tala
        'YER' => '886', // Yemeni Rial
        'ZMK' => '894', // Kwacha
        'TWD' => '901', // New Taiwan Dollar
        'SDG' => '938', // Sudanese Dinar
        'UYI' => '940', // Uruguay Peso en Unidades Indexadas
        'RSD' => '941', // Serbian Dinar
        'MZN' => '943', // Metical
        'AZN' => '944', // Azerbaijanian Manat
        'RON' => '946', // New Leu
        'CHE' => '947', // WIR Euro
        'CHW' => '948', // WIR Franc
        'TRY' => '949', // New Turkish Lira
        'XAF' => '950', // CFA Franc BEAC
        'XCD' => '951', // East Caribbean Dollar
        'XOF' => '952', // CFA Franc BCEAO
        'XPF' => '953', // CFP Franc
        'XBA' => '955', // Bond Markets Units European Composite Unit (EURCO)
        'XBB' => '956', // European Monetary Unit (E.M.U.-6)
        'XBC' => '957', // European Unit of Account 9(E.U.A.-9)
        'XBD' => '958', // European Unit of Account 17(E.U.A.-17)
        'XAU' => '959', // Gold
        'XDR' => '960', // SDR
        'XAG' => '961', // Silver
        'XPT' => '962', // Platinum
        'XTS' => '963', // Codes specifically reserved for testing purposes
        'XPD' => '964', // Palladium
        'SRD' => '968', // Surinam Dollar
        'MGA' => '969', // Malagascy Ariary
        'COU' => '970', // Unidad de Valor Real
        'AFN' => '971', // Afghani
        'TJS' => '972', // Somoni
        'AOA' => '973', // Kwanza
        'BYR' => '974', // Belarussian Ruble
        'BGN' => '975', // Bulgarian Lev
        'CDF' => '976', // Franc Congolais
        'BAM' => '977', // Convertible Marks
        'EUR' => '978', // Euro
        'MXV' => '979', // Mexican Unidad de Inversion (UID)
        'UAH' => '980', // Hryvnia
        'GEL' => '981', // Lari
        'BOV' => '984', // Mvdol
        'PLN' => '985', // Zloty
        'BRL' => '986', // Brazilian Real
        'CLF' => '990', // Unidades de formento
        'USN' => '997', // (Next day)
        'USS' => '998', // (Same day)
        'XXX' => '999' // The codes assigned for transactions where no currency is involved
    );

    protected $_correspCCType           = array(
        'AMEX'   => 'AMEX', // American Express
        'CB'     => 'VISA', // Visa
        'MCVISA' => 'MASTERCARD' // MasterCard
    );

    protected $_availablePaymentMethods = array(
        'DIRECT', 'CPT', 'NX', 'WALLET'
    );

    public $merchantId                  = '';
    public $accessKey                   = '';
    public $proxyHost                   = '';
    public $proxyPort                   = '';
    public $proxyLogin                  = '';
    public $proxyPassword               = '';
    public $environment                 = '';
    public $securityMode                = '';
    public $languageCode                = '';
    public $paymentAction               = '';
    public $paymentMode                 = '';
    public $paymentMethod               = '';
    public $cancelUrl                   = '';
    public $notificationUrl             = '';
    public $returnUrl                   = '';
    public $customPaymentTemplateUrl    = '';
    public $contractNumber              = '';
    public $contractNumberList          = '';
    public $secondaryContractNumberList = '';
    public $customPaymentPageCode       = '';
    public $paymentCurrency             = '';
    public $orderCurrency               = '';
    public $walletId                    = '';
    public $isNewWallet                 = false;
    public $widgetCssUrl                = '';
    public $servletTokenUrl             = '';
    public $widgetJavascriptUrl         = '';

    /**
     * Loaded payment contract types
     * @var array
     */
    protected $_paymentContractType = array();

    /**
     * Is the method code used by Payline?
     * @param string $methodCode Method code
     * @return bool
     */
    public function isPayline($methodCode)
    {
        return strpos(strtolower($methodCode), 'payline') !== false;
    }

    /**
     * Check whether specified currency code is supported
     * @param string $code
     * @return bool
     */
    private function isCurrencyCodeSupported($code)
    {
        return array_key_exists($code, $this->_supportedCurrencyCodes);
    }

    /**
     * Returns the numeric currency code of the chosen currency
     * @param string $currencyCode
     * @return string
     */
    public function getNumericCurrencyCode($alphaCurrencyCode)
    {

        if ($this->isCurrencyCodeSupported($alphaCurrencyCode)) {
            return $this->_supportedCurrencyCodes[$alphaCurrencyCode];
        } else {
            return '0000';
        }
    }

    protected function _hasToSentWalletId()
    {
        return (
            Mage::getStoreConfig('payment/PaylineWALLET/active')
            && (
                Mage::getStoreConfig('payment/PaylineCPT/send_wallet_id')
                || Mage::getStoreConfig('payment/PaylineNX/send_wallet_id')
                || Mage::getStoreConfig('payment/PaylineDirect/send_wallet_id')
            )
        );
    }

    /**
     *
     * Test if we can create a new Wallet
     */
    public function canSubscribeWallet()
    {
        $walletActive               = Mage::getStoreConfig('payment/PaylineWALLET/active');
        $automateSubscriptionEnable = Mage::getStoreConfig('payment/payline_common/automate_wallet_subscription');
        $customer                   = Mage::getSingleton('customer/session')->getCustomer();
        return $walletActive && $automateSubscriptionEnable && $customer->getWalletId() == '';
    }

    public function createWalletForCurrentCustomer($paylineSDK, $array)
    {
        if ($this->canSubscribeWallet()) {
        	try {
        		$customer = Mage::getSingleton('customer/session')->getCustomer();
                $array['contractNumber']     = $array['payment']['contractNumber'];
        		$array['wallet']['walletId'] = Mage::getModel('payline/wallet')->generateWalletId($customer->id); // TODO
                $walletResult                = $paylineSDK->createWallet($array);

		        if (isset($walletResult['result']['code']) && $walletResult['result']['code'] == '02500') {
		                $customer->setWalletId($array['wallet']['walletId'])
		                ->setWalletContractNumber($array['contractNumber'])
		                ->save();
		        } else {
		            $msg = '[createWallet] Error creating';
		            if (is_string($walletResult)) {
		                $msg = ' '.$walletResult;
		            } elseif (isset($walletResult['result']['code'])) {
		                $msg = ', code: '.$walletResult['result']['code'];
		            }
		            Mage::helper('payline/logger')->log($msg);
		        }
        	} catch (Mage_Core_Exception $e) {
        		Mage::logException($e);
        	}
        }
    }

    /**
     *
     * Initialize a payline webservice for payment
     * @param string $paymentMethod (CPT, NX or DIRECT)
     * @param $_numericCurrencyCode If provided, will also initialize currency
     */
    public function initPayline($paymentMethod, $_numericCurrencyCode = null)
    {
        if (!in_array($paymentMethod, $this->_availablePaymentMethods)) {
            return false;
        }
        $this->paymentMethod = $paymentMethod;
        if ($_numericCurrencyCode) {
            $this->paymentCurrency = $_numericCurrencyCode;
            $this->orderCurrency   = $this->paymentCurrency;
        }
        $xmlConfigPath    = 'payment/Payline' . $paymentMethod;
        $commonConfigPath = 'payment/payline_common';

        $this->merchantId = Mage::getStoreConfig($commonConfigPath . '/merchant_id');
        $this->accessKey  = Mage::getStoreConfig($commonConfigPath . '/access_key');
        if (Mage::getStoreConfig($commonConfigPath . '/proxy_host') == '') {
            $this->proxyHost = null;
        } else {
            $this->proxyHost = Mage::getStoreConfig($commonConfigPath . '/proxy_host');
        }
        if (Mage::getStoreConfig($commonConfigPath . '/proxy_port') == '') {
            $this->proxyPort = null;
        } else {
            $this->proxyPort = Mage::getStoreConfig($commonConfigPath . '/proxy_port');
        }
        $this->proxyLogin    = Mage::getStoreConfig($commonConfigPath . '/proxy_login');
        $this->proxyPassword = Mage::getStoreConfig($commonConfigPath . '/proxy_password');
        $this->environment   = Mage::getStoreConfig($commonConfigPath . '/environment');

        $this->securityMode  = self::SECURITY_MODE;
        $this->languageCode  = Mage::getStoreConfig($commonConfigPath . '/language');

        //Wallet :
        //If wallet_id is sent & registered, Payline will offer a checkbox "use previous payment information"
        $customerSession   = Mage::getSingleton('customer/session');
        $hasToSendWalletId = Mage::getStoreConfig($xmlConfigPath . '/send_wallet_id');
        if ($hasToSendWalletId) {
            if ($customerSession->isLoggedIn()) {
                $customer = $customerSession->getCustomer();
                if ($walletId = $customer->getWalletId()) {
                    $this->walletId = $walletId;
                }
            }
        }
        //If wallet_id is sent & NOT registered, Payline will save the wallet
        // if isNewWallet walletId will be sent in privateData, so we'll be able to save it in notifyAction (if payment is OK)
        if (Mage::getStoreConfig('payment/PaylineWALLET/active') && Mage::getStoreConfig('payment/payline_common/automate_wallet_subscription')) {
            if ($customerSession->isLoggedIn()) {
                $customer = $customerSession->getCustomer();
                if (!$customer->getWalletId()) {
                    $this->walletId    = Mage::getModel('payline/wallet')->generateWalletId();
                    $this->isNewWallet = true;
                }
            }
        }

        if ($paymentMethod == 'NX') {
            $this->paymentMode        = 'NX';
            $this->paymentAction      = 101;
            $this->cancelUrl          = Mage::getUrl('payline/index/nxcancel');
            $this->notificationUrl    = Mage::getUrl('payline/index/nxnotif');
            $this->returnUrl          = Mage::getUrl('payline/index/nxreturn');
            $this->contractNumberList = $this->_prepareContractList(true, true);
        } elseif ($paymentMethod == 'CPT') {
            $this->paymentMode        = 'CPT';
            $this->paymentAction      = Mage::getStoreConfig($xmlConfigPath . '/payline_payment_action');
            $this->returnUrl          = Mage::getUrl('payline/index/cptreturn');
            $this->notificationUrl    = $this->returnUrl;
            $this->cancelUrl          = $this->returnUrl;
            $this->contractNumberList = $this->_prepareContractList(true, false);
        } elseif ($paymentMethod == 'WALLET') {//1 clic payment
            $this->paymentMode           = 'CPT';
            $this->paymentAction         = Mage::getStoreConfig($xmlConfigPath . '/payline_payment_action');
            $this->updatePersonalDetails = Mage::getStoreConfig($xmlConfigPath . '/update_personal_details');
            $this->updatePaymentDetails  = Mage::getStoreConfig($xmlConfigPath . '/update_payment_details');
            $this->contractNumberList    = $this->_prepareContractList(true, true);
        } else {//direct
            $this->paymentMode        = 'CPT';
            $this->paymentAction      = Mage::getStoreConfig($xmlConfigPath . '/payline_payment_action');
            $this->cancelUrl          = "";
            $this->notificationUrl    = "";
            $this->returnUrl          = "";
            $this->contractNumberList = $this->_prepareContractList(true, true);
        }
        $this->customPaymentTemplateUrl = Mage::getStoreConfig($xmlConfigPath . '/template_url');

        //primary and secondary contracts number
        $cNumber = explode(';', $this->contractNumberList);
        if (isset($cNumber[0])) {
            $this->contractNumber = $cNumber[0];
        }
        $this->secondaryContractNumberList = $this->_prepareContractList(false, false);

        // default template
        $this->customPaymentPageCode = Mage::getStoreConfig($xmlConfigPath . '/custom_payment_page_code');

        // "responsive" template
        $customTheme = $this->_checkUserAgentAgainstRegexps($xmlConfigPath . '/custom_payment_page_code_ua_regexp');
        if ($customTheme) {
            $this->customPaymentPageCode = $customTheme;
        }

        $payline                           = new Payline\PaylineSDK($this->merchantId, $this->accessKey, $this->proxyHost, $this->proxyPort, $this->proxyLogin, $this->proxyPassword, $this->environment, Mage::getBaseDir('var').'/log/paylineSDK_');
        $payline->returnURL                = $this->returnUrl;
        $payline->cancelURL                = $this->cancelUrl;
        $payline->notificationURL          = $this->notificationUrl;
        $payline->customPaymentTemplateURL = $this->customPaymentTemplateUrl;
        $payline->languageCode             = $this->languageCode;
        $payline->customPaymentPageCode    = $this->customPaymentPageCode;
        $payline->securityMode             = $this->securityMode;
        $payline->paymentAction            = $this->paymentAction;

        $payline->usedBy('Magento ' . Mage::getVersion() . ' - v' . Mage::getConfig()->getModuleConfig('Monext_Payline')->version);

        return $payline;
    }

    public function getServletTokenUrl(){
        if(!$this->servletTokenUrl) {
            $this->getConfigWidget();
        }
        return $this->servletTokenUrl;
    }

    public function getWidgetJavascriptUrl(){
        if(!$this->widgetJavascriptUrl) {
            $this->getConfigWidget();
        }
        return $this->widgetJavascriptUrl;
    }

    public function getWidgetCssUrl(){
        if(!$this->widgetCssUrl) {
            $this->getConfigWidget();
        }
        return $this->widgetCssUrl;
    }

    public function addReturnUrls(&$array)
    {
        $array['returnURL'] = $this->returnUrl;
        $array['cancelURL'] = $this->cancelUrl;
        $array['notificationURL'] = $this->notificationUrl;
        return $this;
    }

    protected function getConfigWidget()
    {
        $environment = (!$this->environment)? Mage::getStoreConfig('payment/payline_common/environment') : $this->environment;
        if(strcmp($environment,Payline\PaylineSDK::ENV_DEV)==0){
            $this->servletTokenUrl = Payline\PaylineSDK::DEV_GET_TOKEN_SERVLET;
            $this->widgetJavascriptUrl = Payline\PaylineSDK::DEV_WDGT_JS;
            $this->widgetCssUrl = Payline\PaylineSDK::DEV_WDGT_CSS;
        }elseif(strcmp($environment,Payline\PaylineSDK::ENV_PROD)==0){
            $this->servletTokenUrl = Payline\PaylineSDK::PROD_GET_TOKEN_SERVLET;
            $this->widgetJavascriptUrl = Payline\PaylineSDK::PROD_WDGT_JS;
            $this->widgetCssUrl = Payline\PaylineSDK::PROD_WDGT_CSS;
        }elseif(strcmp($environment,Payline\PaylineSDK::ENV_HOMO)==0){
            $this->servletTokenUrl = Payline\PaylineSDK::HOMO_GET_TOKEN_SERVLET;
            $this->widgetJavascriptUrl = Payline\PaylineSDK::HOMO_WDGT_JS;
            $this->widgetCssUrl = Payline\PaylineSDK::HOMO_WDGT_CSS;
        }
    }

    public function isProduction()
    {
        return Mage::getStoreConfig('payment/payline_common/environment') == Payline\PaylineSDK::ENV_PROD;
    }

    protected function _prepareContractList($primary = true, $filterCB = false)
    {
        $contractList = array();

        $currentStoreId = Mage::app()->getStore()->getId();

        $contracts = Mage::getModel('payline/contract')->getCollection();
        if ($primary) {
            $contracts->addFilterStatus(true, $currentStoreId);
        } else {
            $contracts->addFilterStatus(false, $currentStoreId);
        }

        if ($filterCB) {
            $contracts->addFieldToFilter('contract_type', array('in' => array('CB', 'AMEX', 'MCVISA')));
        }

        foreach ($contracts as $contract) {
            $contractList[] = $contract->getNumber();
        }

        return implode(';', $contractList);
    }

    /**
     * Translate a credit card type from Mage to Payline
     * @param string $MageCCType
     */
    public function transcoCCType($MageCCType)
    {
        return $this->_correspCCType[$MageCCType];
    }

    /**
     * Create the invoice when the customer is redirected to the shop
     */
    public function automateCreateInvoiceAtShopReturn($mode, $order)
    {
        $action           = Mage::getStoreConfig('payment/Payline' . $mode . '/payline_payment_action');
        $canCreateInvoice = Mage::getStoreConfig('payment/Payline' . $mode . '/automate_invoice_creation');
        if ($mode == 'NX') {
            $action = Monext_Payline_Model_Cpt::ACTION_AUTH_CAPTURE;
        }
        if ($canCreateInvoice == self::CREATE_INVOICE_SHOP_RETURN) {
            $this->createInvoice($action, $order);
        }
    }

    public function createInvoice($action, $order)
    {
        $invoice = $order->prepareInvoice();
        if ($action == Monext_Payline_Model_Cpt::ACTION_AUTH) {
            $invoice->setRequestedCaptureCase(Mage_Sales_Model_Order_Invoice::CAPTURE_ONLINE);
        } else {
            $invoice->setRequestedCaptureCase(Mage_Sales_Model_Order_Invoice::CAPTURE_OFFLINE);
        }
        $invoice->register();
        $order->setIsInProcess(true);
        try {
            $transactionSave = Mage::getModel('core/resource_transaction');
            $transactionSave->addObject($order)
                ->addObject($invoice)
                ->save();
        } catch (Mage_Core_Exception $e) {
            Mage::logException($e);
            Mage::helper('payline/logger')->log('[automateCreateInvoiceAtShopReturn] '
                . $order->getIncrementId()
                . ' unable to save the invoice'
            );
        }
    }

    /**
     *
     * Get the contract number from a transaction (needen if defautl contract number!= of the one trully used - fore instance in a web payment & card AMEX)
     * @param PaylineSDK $paylineSDK
     * @param string $transactionId
     * @param string $orderRef
     */
    public function getTransactionContractNumber($paylineSDK, $transactionId, $orderRef)
    {
        $arrayTransDetails = array(
            'archiveSearch'      => '',
            'transactionId'      => $transactionId,
            'orderRef'           => $orderRef,
            'startDate'          => '',
            'endDate'            => '',
            'transactionHistory' => '',
            'version'            => Monext_Payline_Helper_Data::VERSION
        );
        $result            = $paylineSDK->getTransactionDetails($arrayTransDetails);
        if (is_string($result)) {
            Mage::helper('payline/logger')->log('[getTransactionContractNumber] ' . $result);
            return '';
        } elseif (isset($result['result']) && $result['result']['code'] != '0000' && $result['result']['code'] != '2500' && $result['result']['code'] != '04003') {
            //Back to default
            Mage::helper('payline/logger')->log('[getTransactionContractNumber] ' .
                'Error while retrieving transaction contract number for transactionId' . ' ' . $transactionId . ' and order ' . $orderRef . ' error : ' . $result['result']['shortMessage']);
            return '';
        } else {
            return $result['payment']['contractNumber'];
        }
    }

    public function doReauthorization($orderIncId)
    {
        // I think that this method is never use. @jacquesbh 2014-04-12

        $order = Mage::getModel('sales/order')->loadByIncrementId($orderIncId);
        if ($order->getId()) {
            $payment       = $order->getPayment();
            $paymentMethod = $payment->getMethod();
            if (stripos($paymentMethod, 'payline') !== false) {
                $mode       = explode('Payline', $paymentMethod);
                $array      = array();
                $paylineSDK = Mage::helper('payline')->initPayline($mode[1], Mage::helper('payline')->getNumericCurrencyCode($order->getBaseCurrencyCode()));

                // PAYMENT
                $array['payment']['amount']         = round($order->getBaseGrandTotal() * 100);
                $array['payment']['currency']       = Mage::helper('payline')->getNumericCurrencyCode($order->getBaseCurrencyCode());
                $array['payment']['action']         = Monext_Payline_Model_Cpt::ACTION_AUTH;
                $array['payment']['mode']           = 'CPT';
                $array['payment']['contractNumber'] = $this->getTransactionContractNumber($paylineSDK, $payment->getCcTransId(), $orderIncId);
                // TRANSACTION INFO
                $array['transactionID']             = $payment->getCcTransId();
                // ORDER
                $array['order']['ref']              = substr($orderIncId, 0, 50);
                $array['order']['amount']           = $array['payment']['amount'];
                $array['order']['currency']         = $array['payment']['currency'];

                // PRIVATE DATA
                $privateData          = array();
                $privateData['key']   = "orderRef";
                $privateData['value'] = $orderIncId;
                $paylineSDK->addPrivateData($privateData);
                try {
                    $response = $paylineSDK->doReAuthorization($array);
                    if (isset($response['result']) && isset($response['result']['code']) && $response['result']['code'] != '00000') {
                        $errorMessage = Mage::helper('payline')->__("PAYLINE - Capture error") . ": ";
                        $errorMessage .= isset($response['result']['longMessage']) ? $response['result']['longMessage'] : '';
                        $errorMessage .= isset($response['result']['code']) ? " (code " . $response['result']['code'] . ")<br/>" : '';
                        Mage::helper('payline/Logger')->log('[doReauthorization] [' . $orderIncId . '] ' . $errorMessage);
                        Mage::throwException($errorMessage);
                    } else {
                        $payment->setCcTransId($response['transaction']['id']);
                        $transaction = Mage::getModel('sales/order_payment_transaction');
                        $transaction->setOrder($order);
                        $transaction->setOrderPaymentObject($payment)
                            ->loadByTxnId($response['transaction']['id']);
                        $transaction->setTxnId($response['transaction']['id']);
                        $transaction->setTxnType(Mage_Sales_Model_Order_Payment_Transaction::TYPE_CAPTURE);
                        $transaction->save();
                        $payment->save();
                    }
                } catch (Exception $e) {
                    Mage::logException($e);
                }
            }
        } else {
            Mage::throwException($this->__("The order #%s doesn't exist", $orderIncId));
        }
    }

    public function encodeString($string)
    {
        $string = iconv('UTF-8', "ASCII//TRANSLIT", $string);
        return $string;
    }


    /**
     * @param $string
     * @param int $length
     * @return false|string
     */
    public function encodeStringAndTruncate($string, $length=0)
    {
        $encoded = str_replace(array("\r","\n","\t"), array('','',''), $string);
        $encoded = $this->encodeString($encoded);
        if($length>0) {
            $encoded = substr($encoded,0,$length);
        }

        return $encoded;
    }

    public function setOrderStatus($order, $status)
    {
        $state = Mage::getModel('sales/order_status')->getCollection()
            ->joinStates()
            ->addFieldToFilter('state_table.status', $status)
            ->getFirstItem()
            ->getState();

        if (empty($state)) {
            $state = Mage_Sales_Model_Order::STATE_PROCESSING;
        }
        $order->setState($state, $status);
    }

    protected function _checkUserAgentAgainstRegexps($regexpsConfigPath)
    {
        if (empty($_SERVER['HTTP_USER_AGENT'])) {
            return false;
        }

        $configValueSerialized = Mage::getStoreConfig($regexpsConfigPath, Mage::app()->getStore());

        if (!$configValueSerialized) {
            return false;
        }

        $regexps = @unserialize($configValueSerialized);

        if (empty($regexps)) {
            return false;
        }

        return self::getPackageByUserAgent($regexps, $regexpsConfigPath);
    }

    public static function getPackageByUserAgent(array $rules)
    {
        foreach ($rules as $rule) {

            $regexp = '/' . trim($rule['regexp'], '/') . '/';

            if (@preg_match($regexp, $_SERVER['HTTP_USER_AGENT'])) {
                return $rule['value'];
            }
        }

        return false;
    }

    public function getPaymentContractType($methodCode, $selector)
    {
        if (empty($methodCode)) {
            return '';
        }

        if (!isset($this->_paymentContractType[$methodCode])) {
            $this->_paymentContractType[$methodCode] = array();
        }
        if (!isset($this->_paymentContractType[$methodCode][$selector])) {
            $name = '';
            switch ($methodCode) {
                case 'PaylineDIRECT':
                    $paymentContract = Mage::getModel('payline/contract')
                        ->load($selector);
                    $name = $paymentContract->getName();
                    break;

                default:
                    $paymentContract = Mage::getModel('payline/contract')
                        ->getCollection()
                        ->addFieldToFilter('number', $selector)
                        ->getFirstItem();
                    $name = $paymentContract->getContractType();
            }
            $this->_paymentContractType[$methodCode][$selector] = $name;
        } else {
            $name = $this->_paymentContractType[$methodCode][$selector];
        }
        return $name;
    }

    /**
     * @param $order Order object
     * @param $paymentMode Monext_Payline_Model_Cpt::ACTION_AUTH / ACTION_AUTH_CAPTURE
     * @param bool $isReAuth true if re-authorization, false if not
     */
    public function setOrderStatusAccordingToPaymentMode($order, $paymentMode, $isReAuth = false)
    {
        $paymentMethod       = $order->getPayment()->getMethod();
        $reauthOrderStatus   = Mage::getStoreConfig('payment/payline_common/reauthorized_order_status');
        $capturedOrderStatus = Mage::getStoreConfig('payment/payline_common/captured_order_status');
        $authOrderStatus     = Mage::getStoreConfig('payment/payline_common/authorized_order_status');

        /* Monext_Payline_Model_Cpt::ACTION_RE_AUTH stand for reauth and reauth+capture.
         * So set the exact transaction type in order to set the right order status. */
        if ($paymentMode == Monext_Payline_Model_Cpt::ACTION_RE_AUTH) {
            $reauthAction = Mage::getStoreConfig('payment/' . $paymentMethod . '/action_when_order_seven_day_old');
            $paymentMode  = $reauthAction;
        }
        $arrayReAuth = array(
            Monext_Payline_Model_Cpt::ACTION_AUTH         => $reauthOrderStatus,
            Monext_Payline_Model_Cpt::ACTION_AUTH_CAPTURE => $capturedOrderStatus,
            Monext_Payline_Model_Cpt::ACTION_CAPTURE      => $capturedOrderStatus,
        );
        $arrayAuth   = array(
            Monext_Payline_Model_Cpt::ACTION_AUTH         => $authOrderStatus,
            Monext_Payline_Model_Cpt::ACTION_AUTH_CAPTURE => $capturedOrderStatus,
            Monext_Payline_Model_Cpt::ACTION_CAPTURE      => $capturedOrderStatus
        );

        $errorTxt = '[Monext_Payline_Helper_Data#setOrderStatusAccordingToPaymentMode]'
                . ' ERROR payment ' . $paymentMode . ' action not found. Unable to set order status';
        if ($isReAuth) {
            if (isset($arrayReAuth[$paymentMode])) {
                $conf = $arrayReAuth[$paymentMode];
            } else {
                Mage::log($errorTxt);
                return;
            }
        } else {
            if (isset($arrayAuth[$paymentMode])) {
                $conf = $arrayAuth[$paymentMode];
            } else {
                Mage::log($errorTxt);
                return;
            }
        }
        $this->setOrderStatus($order, $conf);
    }

    /**
     * Check if transaction exist
     * @param array Can contain following keys : transaction_id, txn_id
     * @return false if transaction doesn't exist, otherwise return the transaction model
     */
    public function transactionExist($params)
    {
        if (isset($params['txn_id'])) {
            $transaction = Mage::getModel('sales/order_payment_transaction')
            ->getCollection()
            ->addFieldToFilter('txn_id', $params['txn_id'])
            ->getFirstItem();
        } elseif (isset($params['transaction_id'])) {
            $transaction = Mage::getModel('sales/order_payment_transaction')->load($params['transaction_id']);
        }
        if ($transaction->getId()) {
            return $transaction;
        }
        return false;
    }

    /**
     * Return all contract with is_included_wallet_list db columns set to true.
     * Filtered by store_id and website_id
     * @param $store_id {int} can be null
     * @param $website_id {int} can be null
     * @return collection of matched contracts
     */
    public function getContractWalletList($store_id = null, $website_id = null, $isDefaultLevel = false)
    {
        if ($isDefaultLevel) {
            $collection = Mage::getModel('payline/contract')->getCollection()->addFieldToFilter('is_included_wallet_list', 1);
        } else {
            $collection = Mage::getModel('payline/contract')->getCollection();
            if (isset($store_id)) {
                $collection->addStoreFilter($store_id);
            }
            if (isset($website_id)) {
                $collection->addWebsiteFilter($website_id);
            }
            $collection->getSelect()->where(
                new Zend_Db_Expr('IFNULL(status.is_included_wallet_list,main_table.is_included_wallet_list)') . '=1');
        }
        return $collection;
    }

    /**
     * Build a contract number wallet array used by doWebPayment & createWebWallet
     * @return array An array of contracts number or an null if not found
     */
    public function buildContractNumberWalletList()
    {
        $storeId            = Mage::app()->getStore()->getStoreId();
        $contractCollection = $this->getContractWalletList($storeId);
        $contractsNumber    = array();
        foreach ($contractCollection as $contract) {
            $contractsNumber[] = $contract['number'];
        }
        return sizeof($contractsNumber) > 0 ? $contractsNumber : null;
    }

    public function getProxyParams(){
    	return array('host' => $this->proxyHost, 'port' => $this->proxyPort, 'login' => $this->proxyLogin, 'password' => $this->proxyPassword);
    }

    /**
     *
     * Sets order details in Payline request, from cart content
     * @param \Payline\PaylineSDK $paylineSDK the current sdk instance
     * @param $order the current order
     * @param $sendPaylineproductCat bool flag to determine wether store or Payline product categories shall be sent
     * @param $pcCol Payline product categories collection
     */
    public function setOrderDetails($paylineSDK, $order, $sendPaylineproductCat, $pcCol){
    	$items = $order->getAllItems();
    	if ($items) {
    		if(count($items)>100) $items=array_slice($items,0,100);
    		foreach($items as $item) {
    			$itemPrice = round($item->getPrice()*100);
    			if($itemPrice > 0){
    				$product = array();
    				$itemProduct = Mage::getModel('catalog/product')->load($item->getProductId());
    				$catIdsArray = $itemProduct->getCategoryIds();
    				$currentProductCategoryId = $catIdsArray[0];
    				$currentProductCategory = Mage::getModel('catalog/category')->load($currentProductCategoryId);
    				if($currentProductCategory != null){
    					$currentProductCategoryData = $currentProductCategory->getData();
    					$productCategoryTree = explode('/',$currentProductCategoryData['path']);
    					$productCategoryDepth = sizeof($productCategoryTree);
    					$categorySet = false;
    					$subcategory1Set = false;
    					$subcategory2Set = false;
    					for($c=0;$c<$productCategoryDepth-1;$c++){
    						$productCategory = Mage::getModel('catalog/category')->load($productCategoryTree[$c]);
    						$productCategoryData = $productCategory->getData();
    						if(!in_array($productCategoryData['entity_id'],array(1,2))){
    							// skip "Root Catalog" and "Root Catalog/Default Category/"
    							if(!$categorySet){
    								if($sendPaylineproductCat){
    									$product['category'] = $pcCol->getAssignedPaylineCatId($productCategoryData['entity_id']);
    								}else{
    									$product['category'] = $productCategoryData['name'];
    								}
    								$categorySet = true;
    							}elseif(!$subcategory1Set){
    								if($sendPaylineproductCat){
    									$product['subcategory1'] = $pcCol->getAssignedPaylineCatId($productCategoryData['entity_id']);
    								}else{
    									$product['subcategory1'] = $productCategoryData['name'];
    								}
    								$subcategory1Set = true;
    							}elseif(!$subcategory2Set){
    								if($subcategory2Set){
    									$product['subcategory2'] = $pcCol->getAssignedPaylineCatId($productCategoryData['entity_id']);
    								}else{
    									$product['subcategory2'] = $productCategoryData['name'];
    								}
    								$subcategory2Set = true;
    							}
    						}
    					}
    				}
    				// delete special characters
    				if($categorySet){
    				    $product['category'] = preg_replace('/[^A-Za-z0-9\-]/', '', substr(str_replace(array("\r","\n","\t"," "), array('','','',''),$product['category']),0,50));
    				}else{
    				    $product['category'] = null;
    				}
    				if($subcategory1Set){
    				    $product['subcategory1'] = preg_replace('/[^A-Za-z0-9\-]/', '', substr(str_replace(array("\r","\n","\t"," "), array('','','',''),$product['subcategory1']),0,50));
    				}else{
    				    $product['subcategory1'] = null;
    				}
    				if($subcategory2Set){
    				    $product['subcategory2'] = preg_replace('/[^A-Za-z0-9\-]/', '', substr(str_replace(array("\r","\n","\t"," "), array('','','',''),$product['subcategory2']),0,50));
    				}else{
    				    $product['subcategory2'] = null;
    				}

                    $product['ref'] = $this->encodeStringAndTruncate($item->getName(),50);

                    $product['price'] = round($item->getPrice()*100);
    				$product['quantity'] = round($item->getQtyOrdered());

                    $product['comment'] = $this->encodeStringAndTruncate($itemProduct->getDescription(),255);
                    //For Oney Comment is a mandatory field
                    if(empty($product['comment'])) {
                        $product['comment'] = $product['ref'] ? $product['ref'] : $item->getSku();
                    }

    				$product['taxRate'] = round($item->getTaxPercent()*100);

                    $product['additionalData'] = $this->encodeStringAndTruncate($item->getAdditionalData(),255);

    				$product['brand'] = $itemProduct->getAttributeText('manufacturer');
    				$paylineSDK->addOrderDetail($product);
    			}
    			continue;
    		}
    	}
    }

    /**
     * Return all contract with is_secure set to $securized.
     *
     * @param bool $securized
     *
     * @return Monext_Payline_Model_Mysql4_Contract_Collection
     */
    public function getCcContracts($securized=true, $cbType= true, $primary=true)
    {
        $key = ($securized) ? 1 : 0;
        $keyPrimary = ($primary) ? 1 : 0;
        if (!isset($this->_ccContracts[$key]) || !isset($this->_ccContracts[$key][$keyPrimary])) {

            $contracts = Mage::getModel('payline/contract')->getCollection()
                ->addFilterSecure($securized, Mage::app()->getStore()->getId());

            if($cbType) {
                $contracts->addFieldToFilter('contract_type', array('CB', 'AMEX', 'MCVISA'));
            }

            //No need to be primary nor secondary if contract is secure
            if(!$securized) {
                $contracts->addFilterStatus($primary, Mage::app()->getStore()->getId());
            }

            $this->_ccContracts[$key][$keyPrimary] = $contracts;
        }

        return $this->_ccContracts[$key][$keyPrimary];
    }

    /**
     * Return contract by code en securise property
     *
     * @param string $type
     * @param bool $securized
     *
     * @return Monext_Payline_Model_Contract
     */
    public function getContractByType($type, $securized=false)
    {
        $returnContract=false;
        $contractNumber=0;
        $contracts = $this->getCcContracts($securized);
        foreach ($contracts as $contract) {
            if($type==$contract->getContractType()) {
                $returnContract = $contract;
                $contractNumber++;
            }
        }
        if($contractNumber==1 and $returnContract) {
            return $returnContract;
        } else {
            return false;
        }
    }


    /**
     * @return string
     */
    public function getWeb2TokenKey()
    {
        return Mage::getStoreConfig('payment/PaylineDIRECT/web2token_key');
    }


    /**
     * @return string
     */
    protected function _getAes256()
    {
        return hash("SHA256", $this->accessKey, true);
    }

    /**
     * Reserve an orderid and return a unique crypted key
     *
     * @return string
     */
    public function getCryptedKeys($quote=null)
    {
        //Reserve an OrderId
        if(is_null($quote)) {
            $quote= Mage::getSingleton('checkout/session')->getQuote();
        }

        $quote->reserveOrderId()->save();
        $crypted = array();

        $numericCurrency =  $this->getNumericCurrencyCode($quote->getBaseCurrencyCode());

        $paylineSDK = $this->initPayline('DIRECT',$numericCurrency);

        $orderRef = $quote->getReservedOrderId();

        $availableContracts = $this->getCcContracts();

        foreach($availableContracts as $contract) {
            $messageUtf8 = utf8_encode($this->merchantId . ";" . $orderRef . ";" . $contract['number']);
            $crypted[$contract['id']] = $paylineSDK->getEncrypt($messageUtf8, $this->_getAes256());
        }

        return $crypted;
    }

    /**
     *
     * @param string $tokenData
     * @param string $order
     *
     * @return array
     */
    public function getDecryptedCardTokenPan($tokenData, $order)
    {
        $numericCurrency =  $this->getNumericCurrencyCode($order->getBaseCurrencyCode());

        $paylineSDK = $this->initPayline('DIRECT',$numericCurrency);

        $decrypt = $paylineSDK->gzdecode($paylineSDK->getDecrypt($tokenData, $this->_getAes256()));
        $arrayDecrypt = explode(';',$decrypt);
        $paymentData = array(
                'cardTokenPan'	=> $arrayDecrypt[0],
                'cardExp'		=> $arrayDecrypt[1],
                'vCVV'			=> $arrayDecrypt[2],
                'orderRef'		=> $arrayDecrypt[3],
                'cardType'		=> $arrayDecrypt[4],
                'cardIsCVD '	=> $arrayDecrypt[5],
                'cardCountry'	=> $arrayDecrypt[6],
                'cardProduct'	=> $arrayDecrypt[7],
                'bankCode'		=> $arrayDecrypt[8]
        );

        return $paymentData;
    }


    public function getCptConfigTemplate()
    {
        return Mage::getStoreConfig('payment/PaylineCPT/data_template', Mage::app()->getStore());
    }

    public function disableOnepagePaymentStep()
    {
        $template = $this->getCptConfigTemplate();
        return !empty($template) && $template!='redirect';
    }


    public function getContractsForWidget($primary=true)
    {
        $contractCPT = $this->getCcContracts(false, false,$primary);

        $contracts = array();
        foreach($contractCPT as $contract) {
            $contracts[]= $contract->getNumber();
        }

        return $contracts;
    }


    public function getDefaultContractNumberForWidget()
    {
        $contracts = $this->getContractsForWidget();

        return !empty($contracts) ? $contracts[0] : false;
    }


} // end class
