<?php

/**
 * This controller manage primary and secondary payline contracts in back office
 */
class Monext_Payline_Adminhtml_Payline_ManagecontractsController extends Mage_Adminhtml_Controller_Action
{

    protected function _isAllowed()
    {
        return Mage::getSingleton('admin/session')->isAllowed('payline/contract');
    }

    public function indexAction()
    {
        $this->_title($this->__('Manage Payline Contracts'));
        $this->loadLayout();
        $this->_setActiveMenu('system');
        $this->renderLayout();
    }

    public function importFromConfigAction()
    {
        return $this->_importContract('adminhtml/system_config/edit/section/payline');
    }

    public function importAction()
    {
        return $this->_importContract('*/*');
    }


    protected function _importContract($utlPath='*/*')
    {
        $updatedAt = new Zend_Date();
        $updatedAt = $updatedAt->toString(Varien_Date::DATETIME_INTERNAL_FORMAT);
        try {
            /** @var PaylineSDK $paylineSDK */
            $paylineSDK = Mage::helper('payline')->initPayline('CPT');
            $result = $paylineSDK->getMerchantSettings(array('version' => Monext_Payline_Helper_Data::VERSION));
        } catch (Exception $e) {
            Mage::logException($e);
            $msg    = Mage::helper('payline')->__('Error during import');
            Mage::getSingleton('adminhtml/session')->addError($msg);
            $msgLog = 'Unknown PAYLINE ERROR (payline unreachable?)';
            Mage::helper('payline/logger')->log('[importAction] ' . $this->order->getIncrementId() . $msgLog);
            $this->_redirect($utlPath);
            return;
        }

        if (is_string($result)) { //error message from web service
            $msgError = $result;
            if (stristr('Authorization Required', $result)) {
                $msgError .= '. ' . Mage::helper('payline')->__('Please check your authentication params (merchant Id / access key / environment) in System > Configuration');
            } elseif (stristr('could not connect to host', $result)) {
                $msgError .= '. ' . Mage::helper('payline')->__('Please check your proxy params (Proxy host / port / login / password) in System > Configuration');
            }

            Mage::getSingleton('adminhtml/session')->addError($msgError);
            Mage::helper('payline/logger')->log('[importAction] ' . $result);
        } elseif (isset($result['listPointOfSell']['pointOfSell'])) {
            $nbContracts     = Mage::getModel('payline/contract')->getCollection()->getSize();
            $first           = true;
            $listPointOfSell = array();
            $listPointOfSell = $result['listPointOfSell']['pointOfSell'];

            //With only point of sell need to change structure
            if(!empty($listPointOfSell['contracts'])) {
                $listPointOfSell = array($listPointOfSell);
            }

            foreach ($listPointOfSell as $key => $pointOfSell) {
                if (is_object($pointOfSell)) {
                    try {
                        $contracts        = $pointOfSell->contracts->contract;
                        $pointOfSellLabel = $pointOfSell->label;
                    }catch (\Exception $e) {
                        Mage::helper('payline/logger')->log('[Import contract] ' . $e->getMessage());
                        Mage::helper('payline/logger')->log('[Import contract] ' . json_encode($pointOfSell));
                    }
                } else { //if only one point of sell, we parse an array
                    if (!empty($pointOfSell['contracts'])) {
                        $contracts = !empty($pointOfSell['contracts']['contract']) ? $pointOfSell['contracts']['contract'] : [];
                    }
                    $pointOfSellLabel = (!empty($pointOfSell['label'])) ? $pointOfSell['label']: '';
                }

                $firstKey = key($contracts);
                if(!is_numeric($firstKey) && isset($contracts['contractNumber'])) {
                    $contracts = [$contracts];
                }

                foreach ($contracts as $key => $contract) {
                    $contracts[$key] = (object)$contract;
                }

                if (is_array($contracts) && isset($contracts['contractNumber'])) {
                    $contracts = array((object) $contracts);
                }

                foreach ($contracts as $contract) {
                    $myContract = Mage::getModel('payline/contract')
                            ->getCollection()
                            ->addFieldToFilter('number', $contract->contractNumber)
                            ->getFirstItem();

                    if ($myContract->getId()) { //contract exists, update
                        $myContract->setName($contract->label)
                            ->setPointOfSell($pointOfSellLabel)
                            ->setContractDate($updatedAt)
                            ->setContractType($contract->cardType)
                            ->save();
                    } else {
                        $new_contract = Mage::getModel('payline/contract')
                            ->setName($contract->label)
                            ->setNumber($contract->contractNumber)
                            ->setPointOfSell($pointOfSellLabel)
                            ->setContractType($contract->cardType)
                            ->setContractDate($updatedAt);

                        if ($first && !$nbContracts) { //force to have a primary contract
                            $new_contract->setIsPrimary(1);
                            $first = false;
                        }

                        $new_contract->save();
                    }
                }
            }

            //delete contracts not updated
            $toDelete = Mage::getModel('payline/contract')->getCollection()
                    ->addFieldToFilter('contract_date', array('neq' => $updatedAt));
            foreach ($toDelete as $contract) {
                $contract->delete();
            }

            Mage::getSingleton('adminhtml/session')->addSuccess(Mage::helper('payline')->__('Contracts imported successfully. Don\'t forget to define primary and secondary contracts!'));
        } else {

            $msgError = 'Unknown error occured while importing contract';
            if (!empty($result['result']['longMessage'])) {
                $msgError .= ' ('.$result['result']['longMessage'].')';
            } elseif (!empty($result['result']['code'])) {
                $msgError .= ' (code: '.$result['result']['code'].')';
            }

            Mage::getSingleton('adminhtml/session')->addError($msgError);
            Mage::helper('payline/logger')->log('[importAction] ' . $msgError);

        }
        $this->_redirect($utlPath);
    }

    /**
     * can't create new contract, forward to edit action
     */
    public function newAction()
    {
        $this->_forward('edit');
    }

    public function saveAction()
    {
        if ($this->getRequest()->getPost()) {
            try {
                $postData = $this->getRequest()->getPost();

                // Update existing contract
                if (array_key_exists('id', $postData)) {
                    $contract = Mage::getModel('payline/contract')->load($postData['id']);

                    $is_primary   = (isset($postData['is_primary']) ? $postData['is_primary'] : 0);
                    $is_secondary = (isset($postData['is_secondary']) ? $postData['is_secondary'] : 0);
                    $is_secure = (isset($postData['is_secure']) ? $postData['is_secure'] : 0);

                    $count = Mage::getModel('payline/contract')->getCollection()
                        ->addFieldToFilter('is_primary', 1)
                        ->addFieldToFilter('id', array('neq' => $contract->getId()))
                        ->getSize();
                    //at least one contract must be primary
                    if (!$is_primary && !$count) {
                        Mage::getSingleton('adminhtml/session')->addError(Mage::helper('payline')->__('At leat one contract must be primary'));
                        $this->_redirect('*/*');
                        return;
                    }

                    //primary and secondary contracts must ne defined in only one point of sell
                    //because it's impossible to send to webservice contracts that are not in the same point of sell
                    //Mage::getResourceModel('payline/contract')->removePrimaryAndSecondaryNotIn($contract->getPointOfSell());

                    $contract->setIsPrimary($is_primary)
                        ->setIsSecondary($is_secondary)
                        ->setIsSecure($is_secure)
                        ->save();
                    Mage::getSingleton('adminhtml/session')->addSuccess(Mage::helper('payline')->__('Contract saved successfully'));
                }
            } catch (Exception $e) {
                Mage::getSingleton('adminhtml/session')->addError($e->getMessage());
                $this->_redirect('*/*/edit', array('id' => $this->getRequest()->getParam('id')));
                return;
            }
        }

        $this->_redirect('*/*/');
    }

    public function editAction()
    {
        $contract = Mage::getModel('payline/contract')->load($this->getRequest()->getParam('id', false));

        //checks if the item has been loaded correctly
        if (!$contract->getId()) {
            Mage::getSingleton('adminhtml/session')->addError(Mage::helper('payline')->__("Required contract does not exist"));
            $this->_redirect('*/*/index');
            return;
        }

        // Register model to use later in blocks
        Mage::register('current_contract', $contract);

        $this->loadLayout();
        $this->renderLayout();
    }

    public function massStatusAction()
    {
        $contractIds = (array) $this->getRequest()->getParam('contract');
        $status      = (int) $this->getRequest()->getParam('status');
        $store       = $this->getRequest()->getParam('store', 0);
        $website     = $this->getRequest()->getParam('website', 0);

        Mage::getResourceModel('payline/contract_status')->updateContractStatus($contractIds, $status, $website, $store);

        $this->_redirect('*/*/', array('website' => $website, 'store' => $store));
    }


    public function massWalletAction()
    {
        $contractIds = (array) $this->getRequest()->getParam('contract');
        $subscribe   = (int) $this->getRequest()->getParam('walletList');
        $store       = $this->getRequest()->getParam('store', 0);
        $website     = $this->getRequest()->getParam('website', 0);

        Mage::getResourceModel('payline/contract')->updateContractWalletList(
            $contractIds, $subscribe, $website, $store
        );

        $this->_redirect('*/*/', array('website' => $website, 'store' => $store));
    }


    public function massSecureAction()
    {
        $contractIds = (array) $this->getRequest()->getParam('contract');
        $secure      = (int) $this->getRequest()->getParam('setSecure');
        $store       = $this->getRequest()->getParam('store', 0);
        $website     = $this->getRequest()->getParam('website', 0);

        Mage::getResourceModel('payline/contract')->updateContractSecureList(
                $contractIds, $secure, $website, $store
        );

        $this->_redirect('*/*/', array('website' => $website, 'store' => $store));
    }
    /**
     * contracts grid for AJAX request
     */
    public function gridAction()
    {
        $this->loadLayout();
        $this->renderLayout();
    }

}
